==> dkd-0.5.16.tar.gz: dkd-0.5.16/dkd/content.py <==
# -*- coding: utf-8 -*-
#
#   Dao-Ke-Dao: Universal Message Module
#
#                                Written in 2019 by Moky <albert.moky@gmail.com>
#
# ==============================================================================
# MIT License
#
# Copyright (c) 2019 Albert Moky
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ==============================================================================

import random

from .types import ContentType


def random_positive_integer():
    """
    :return: random integer greater than 0
    """
    return random.randint(1, 2**32-1)


class Content(dict):
    """This class is for creating message content

        Message Content
        ~~~~~~~~~~~~~~~

        data format: {
            'type'    : 0x00,            // message type
            'sn'      : 0,               // serial number

            'group'   : 'Group ID',      // for group message

            //-- message info
            'text'    : 'text',          // for text message
            'command' : 'Command Name',  // for system command
            //...
        }
    """

    # noinspection PyTypeChecker
    def __new__(cls, content: dict):
        """
        Create message content

        :param content: content info
        :return: Content object
        """
        if content is None:
            return None
        elif cls is Content:
            if isinstance(content, Content):
                # return Content object directly
                return content
            # get subclass by message content type
            clazz = message_content_classes.get(int(content['type']))
            if clazz is not None:
                assert issubclass(clazz, Content), '%s must be sub-class of Content' % clazz
                return clazz.__new__(clazz, content)
        # subclass or default Content(dict)
        return super().__new__(cls, content)

    def __init__(self, content: dict):
        if self is content:
            # no need to init again
            return
        super().__init__(content)
        # message type: text, image, ...
        self.__type: ContentType = None
        # serial number: random number to identify message content
        self.__sn: int = None

    # message content type: text, image, ...
    @property
    def type(self) -> ContentType:
        if self.__type is None:
            self.__type = ContentType(int(self['type']))
        return self.__type

    # random number to identify message content
    @property
    def serial_number(self) -> int:
        if self.__sn is None:
            self.__sn = int(self['sn'])
        return self.__sn

    # Group ID/string for group message
    #    if field 'group' exists, it means this is a group message
    @property
    def group(self) -> str:
        return self.get('group')

    @group.setter
    def group(self, value: str):
        if value is None:
            self.pop('group', None)
        else:
            self['group'] = value

    #
    #   Factory
    #
    @classmethod
    def new(cls, content: dict=None, content_type: ContentType=0):
        """
        Create message content with 'type' & 'sn' (serial number)

        :param content:      content info, if empty then create a new one with 'type'
        :param content_type: content type, if not empty then set into content
        :return: Content object
        """
        if content is None:
            # create content with 'sn'
            content = {
                'sn': random_positive_integer(),
            }
        else:
            # generate 'sn'
            if 'sn' not in content:
                content['sn'] = random_positive_integer()
        # set content type
        if content_type > 0:
            content['type'] = content_type
        # new Content(dict)
        return cls(content)


"""
    Message Content Classes Map
"""

message_content_classes = {
}

==> dkd-0.8.0.tar.gz: dkd-0.8.0/dkd/content.py <==
# -*- coding: utf-8 -*-
#
#   Dao-Ke-Dao: Universal Message Module
#
#                                Written in 2019 by Moky <albert.moky@gmail.com>
#
# ==============================================================================
# MIT License
#
# Copyright (c) 2019 Albert Moky
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ==============================================================================

import weakref
from typing import Optional, Generic

from .types import ID


class Content(dict, Generic[ID]):
    """This class is for creating message content

        Message Content
        ~~~~~~~~~~~~~~~

        data format: {
            'type'    : 0x00,            // message type
            'sn'      : 0,               // serial number

            'group'   : 'Group ID',      // for group message

            //-- message info
            'text'    : 'text',          // for text message
            'command' : 'Command Name',  // for system command
            //...
        }
    """

    def __init__(self, content: dict):
        if self is content:
            # no need to init again
            return
        super().__init__(content)
        self.__delegate: weakref.ReferenceType = None
        # lazy
        self.__type: int = None
        self.__sn: int = None
        self.__time: int = None
        self.__group = None

    @property
    def delegate(self):  # Optional[MessageDelegate[ID]]:
        if self.__delegate is not None:
            return self.__delegate()

    @delegate.setter
    def delegate(self, value):
        if value is None:
            self.__delegate = None
        else:
            self.__delegate = weakref.ref(value)

    # message content type: text, image, ...
    @property
    def type(self) -> int:
        if self.__type is None:
            self.__type = int(self['type'])
        return self.__type

    # serial number: random number to identify message content
    @property
    def serial_number(self) -> int:
        if self.__sn is None:
            self.__sn = int(self['sn'])
        return self.__sn

    @property
    def time(self) -> int:
        if self.__time is None:
            time = self.get('time')
            if time is None:
                self.__time = 0
            else:
                self.__time = int(time)
        return self.__time

    # Group ID/string for group message
    #    if field 'group' exists, it means this is a group message
    @property
    def group(self) -> Optional[ID]:
        if self.__group is None:
            self.__group = self.delegate.identifier(string=self.get('group'))
        return self.__group

    @group.setter
    def group(self, value: str):
        if value is None:
            self.pop('group', None)
        else:
            self['group'] = value
        # lazy load
        self.__group = None
